@@ -1,6 +1,7 @@
-/* Clearlooks Engine
+/* Clearlooks theme engine
  * Copyright (C) 2005 Richard Stellingwerff.
- * Copyright (C) 2006 Benjamin Berg
+ * Copyright (C) 2007 Benjamin Berg
+ * Copyright (C) 2007 Andrea Cimitan
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +22,7 @@
  * and by Alexander Larsson <alexl@redhat.com>
  * Modified by Richard Stellingwerff <remenic@gmail.com>
  */
+ 
 #include <gtk/gtkstyle.h>
 
 #ifndef CLEARLOOKS_STYLE_H
@@ -48,8 +50,12 @@ struct _ClearlooksStyle
 	ClearlooksColors colors;
 
 	ClearlooksStyles style;
-	
+
+	guint8   reliefstyle;
 	guint8   menubarstyle;
+	guint8   toolbarstyle;
+	GdkColor focus_color;
+	gboolean has_focus_color;
 	GdkColor scrollbar_color;
 	gboolean colorize_scrollbar;
 	gboolean has_scrollbar_color;
@@ -59,13 +65,11 @@ struct _ClearlooksStyle
 
 struct _ClearlooksStyleClass
 {
-  GtkStyleClass parent_class;
-  
-  ClearlooksStyleFunctions style_functions[CL_NUM_STYLES];
-};
+	GtkStyleClass parent_class;
 
+	ClearlooksStyleFunctions style_functions[CL_NUM_STYLES];
+};
 
 GE_INTERNAL void clearlooks_style_register_type (GTypeModule *module);
 
-
 #endif /* CLEARLOOKS_STYLE_H */
