@@ -213,129 +213,170 @@ paint_menuitem (cairo_t *cr, GtkStyle *style, GtkStateType state_type,
 }
 
 static void
-paint_button (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkShadowType shadow_type,
-	gdouble x, gdouble y, gdouble width, gdouble height)
+paint_button (cairo_t *cr, GtkStyle *style,
+		GtkStateType state_type, GtkShadowType shadow_type,
+		gboolean has_focus, gboolean extra_shadow,
+		gdouble x, gdouble y, gdouble width, gdouble height)
 {
-	gdouble radius = 2.0;
+	gdouble radius = (extra_shadow) ? 3.5 : 2.5;
 	cairo_pattern_t *crp;
-	CairoColor bg, c1, c2, border;
+	CairoColor base, inner_top, inner_bot, grad_top, grad_bot, stroke;
+        gdouble h, s, b;
 
+	if (shadow_type == GTK_SHADOW_NONE)
+		return;
 
-	if (state_type == GTK_STATE_INSENSITIVE)
+	/* calculate colours */
+	ge_gdk_color_to_cairo (&style->bg[state_type], &base);
+	ge_hsb_from_color (&base, &h, &s, &b);
+	ge_color_from_hsb (h, s, b - 0.52, &stroke);
+
+	if (extra_shadow)
 	{
-		/* solid fill for insensitive */
+		/* extra shadow and focus indicator */
+		if (has_focus)
+		{
+			CairoColor c;
+			ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &c);
+			cairo_set_source_rgba (cr, c.r, c.g, c.b, 0.6);
+		}
+		else
+		{
+			if (state_type == GTK_STATE_INSENSITIVE)
+				cairo_set_source_rgba (cr, 0, 0, 0, 0.05);
+			else
+				cairo_set_source_rgba (cr, 0, 0, 0, 0.1);
+		}
 		ge_cairo_rounded_rectangle (cr, x, y, width, height, radius, CR_CORNER_ALL);
-		gdk_cairo_set_source_color (cr, &style->bg[state_type]);
 		cairo_fill (cr);
 
-		/* reduce height so that a shadow appears beneath the button */
-		height -= 1.0;
+		--radius;
+		++x; ++y; width -= 2; height -= 2;
 	}
 
-	/* optimise for line drawing */
-	x += 0.5; y += 0.5;
-	width -= 1.0; height -= 1.0;
-
-	/* calculate start and end colours for the gradient */
-	ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
-	ge_shade_color (&bg, 0.9, &c2); /* darken */
-	ge_shade_color (&bg, 1.1, &c1); /* lighten */
-	ge_shade_color (&bg, OUTLINE_SHADE, &border); /* border */
-
-
-	switch (shadow_type)
+	if (shadow_type == GTK_SHADOW_IN || shadow_type == GTK_SHADOW_ETCHED_IN)
 	{
-		case GTK_SHADOW_NONE:
+		/* outline */
+		if (state_type == GTK_STATE_INSENSITIVE)
+			gdk_cairo_set_source_color (cr, &style->fg [state_type]);
+		else
+			ge_cairo_set_color (cr, &stroke);
+		ge_cairo_rounded_rectangle (cr, x, y, width, height, radius, CR_CORNER_ALL);
+		cairo_fill (cr);
+
+		++x; ++y; width -= 2; height -= 2;
+
+		ge_cairo_rounded_rectangle (cr, x, y, width, height, radius, CR_CORNER_ALL);
+		if (state_type != GTK_STATE_INSENSITIVE)
+		{
+			/* gradient fill */
+			crp = cairo_pattern_create_linear (x, y, x, y +height);
+			ge_color_from_hsb (h, s, b - 0.05, &grad_top);
+			ge_color_from_hsb (h, s, b - 0.2, &grad_bot);
+			cairo_pattern_add_color_stop_rgb (crp, 0.0, grad_top.r, grad_top.g, grad_top.b);
+			cairo_pattern_add_color_stop_rgb (crp, 0.3, grad_top.r, grad_top.g, grad_top.b);
+			cairo_pattern_add_color_stop_rgb (crp, 0.9, grad_bot.r, grad_bot.g, grad_bot.b);
+			cairo_pattern_add_color_stop_rgb (crp, 1.0, grad_bot.r, grad_bot.g, grad_bot.b);
+			cairo_set_source (cr, crp);
+			cairo_fill (cr);
+			cairo_pattern_destroy (crp);
+		}
+		else
+		{
+			ge_cairo_set_color (cr, &base);
+			cairo_fill (cr);
+		}
+
+		x += 0.5; y += 0.5; height--; width--;
+
+		/* inside shadow */
+		--radius;
+		cairo_move_to (cr, x, y + height);
+		cairo_arc (cr, x + radius, y + radius, radius, G_PI, G_PI * 1.5);
+		cairo_line_to (cr, x + width, y);
+		if (state_type == GTK_STATE_INSENSITIVE)
+			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.1);
+		else
+			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2);
+		cairo_stroke (cr);
+
+		cairo_move_to (cr, x + 1.0, y + height);
+		cairo_line_to (cr, x + 1.0, y + 1);
+		cairo_line_to (cr, x + width, y + 1);
+		cairo_line_to (cr, x + width, y + height);
+		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.05);
+		cairo_stroke (cr);
+
+
+		if (state_type == GTK_STATE_INSENSITIVE)
 			return;
-		case GTK_SHADOW_IN:
-		case GTK_SHADOW_ETCHED_IN:
+
+		cairo_rectangle (cr, x + 1.0, y + 1.0, width - 2, height - 1);
+		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.03);
+		cairo_stroke (cr);
+		return;
+	}
+
+	if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
+	{
+		if (state_type == GTK_STATE_INSENSITIVE)
+		{
+			/* disabled gray stroke */
 			ge_cairo_rounded_rectangle (cr, x, y, width, height, radius, CR_CORNER_ALL);
-			if (state_type != GTK_STATE_INSENSITIVE)
-			{
-				/* gradient fill */
-				crp = cairo_pattern_create_linear (x, y, x, y +height);
-				/*
-				 * cairo_pattern_add_color_stop_rgb (crp, 0.0, 186/255.0, 189/255.0, 182/255.0);
-				 * cairo_pattern_add_color_stop_rgb (crp, 1.0, 136/255.0, 138/255.0, 133/255.0);
-				 */
+			gdk_cairo_set_source_color (cr, &style->fg [state_type]);
+			cairo_fill (cr);
 
-				cairo_pattern_add_color_stop_rgb (crp, 0.0, c1.r, c1.g, c1.b);
-				cairo_pattern_add_color_stop_rgb (crp, 1.0, c2.r, c2.g, c2.b);
-				cairo_set_source (cr, crp);
-				cairo_fill_preserve (cr);
-				cairo_pattern_destroy (crp);
-			}
+			--radius;
+			++x; ++y; width -= 2; height -= 2;
 
-			/* outline */
-			if (state_type == GTK_STATE_INSENSITIVE)
-			{
-				/* disabled gray stroke */
-				gdk_cairo_set_source_color (cr, &style->fg [state_type]);
-			}
-			else
-			{
-				/* dark gray */
-				ge_cairo_set_color (cr, &border);
-			}
-			cairo_stroke (cr);
+			ge_cairo_rounded_rectangle (cr, x, y, width, height , radius, CR_CORNER_ALL);
+			gdk_cairo_set_source_color (cr, &style->bg [state_type]);
+			cairo_fill (cr);
 
-			/* inside shadow */
-			radius = 1.0;
-			cairo_move_to (cr, x + 1.0, y + height - 1.0);
-			cairo_arc (cr, x + radius + 1.0, y + radius + 1.0, radius, G_PI, G_PI * 1.5);
-			cairo_line_to (cr, x + width - 1.0, y + 1.0);
-			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.22);
-			cairo_stroke (cr);
+			return;
+		}
 
-			cairo_move_to (cr, x + 2.0, y + height - 1.0);
-			cairo_line_to (cr, x + 2.0, y + 2.0);
-			cairo_line_to (cr, x + width - 1.0, y + 2.0);
-			cairo_line_to (cr, x + width - 1.0, y + height - 1.0);
-			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.12);
-			cairo_stroke (cr);
 
-			cairo_rectangle (cr, x + 3.0, y + 3.0, width - 5.0, height - 4.0);
-			cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.03);
-			cairo_stroke (cr);
-			break;
-		case GTK_SHADOW_OUT:
-		case GTK_SHADOW_ETCHED_OUT:
-			if (state_type == GTK_STATE_INSENSITIVE)
-			{
-				ge_cairo_rounded_rectangle (cr, x, y, width, height , radius, CR_CORNER_ALL);
-				/* disabled gray stroke */
-				gdk_cairo_set_source_color (cr, &style->fg [state_type]);
-				cairo_stroke (cr);
-			}
-			else
-			{
-				ge_cairo_rounded_rectangle (cr, x, y, width, height, radius, CR_CORNER_ALL);
+		ge_color_from_hsb (h, s, b + 0.156, &inner_top);
+		ge_color_from_hsb (h, s, b, &inner_bot);
+		ge_color_from_hsb (h, s, b + 0.117, &grad_top);
+		ge_color_from_hsb (h, s, b - 0.117, &grad_bot);
 
-				crp = cairo_pattern_create_linear (x, y+2.0, x, y+height-2.0);
+		/* border */
+		ge_cairo_rounded_rectangle (cr, x, y, width, height, radius, CR_CORNER_ALL);
+		ge_cairo_set_color (cr, &stroke);
+		cairo_fill (cr);
 
-				cairo_pattern_add_color_stop_rgb (crp, 0.0, c1.r, c1.g, c1.b);
-				cairo_pattern_add_color_stop_rgb (crp, 1.0, c2.r, c2.g, c2.b);
-				/*
-				cairo_pattern_add_color_stop_rgb (crp, 0.0, 238/255.0, 238/255.0, 236/255.0);
-				cairo_pattern_add_color_stop_rgb (crp, 1.0, 186/255.0, 189/255.0, 182/255.0);
-				*/
-				cairo_set_source (cr, crp);
-				cairo_fill_preserve (cr);
-				cairo_pattern_destroy (crp);
 
-				/* border */
-				ge_cairo_set_color (cr, &border);
-				cairo_stroke (cr);
+		/* inner stroke */
+		++x; ++y;
+		width -= 2; height -= 2;
+		--radius;
 
-				ge_cairo_rounded_rectangle (cr, x+1.0, y+1.0, width-2.0, height-2.0, radius, CR_CORNER_ALL);
-				crp = cairo_pattern_create_linear (x, y, x, y + height);
-				cairo_pattern_add_color_stop_rgba (crp, 0.0, 1.0, 1.0, 1.0, 0.4);
-				cairo_pattern_add_color_stop_rgba (crp, 1.0, 0.8, 0.8, 0.8, 0.4);
-				cairo_set_source (cr, crp);
-				cairo_stroke (cr);
-				cairo_pattern_destroy (crp);
-			}
-			break;
+		crp = cairo_pattern_create_linear (x, y, x, y + height);
+		cairo_pattern_add_color_stop_rgb (crp, 0.0, inner_top.r, inner_top.g, inner_top.b);
+		cairo_pattern_add_color_stop_rgb (crp, 1.0, inner_bot.r, inner_bot.g, inner_bot.b);
+		cairo_set_source (cr, crp);
+
+		ge_cairo_rounded_rectangle (cr, x, y, width, height, radius, CR_CORNER_ALL);
+		cairo_fill (cr);
+		cairo_pattern_destroy (crp);
+
+
+		/* gradient */
+		x+=.5; y+=.5;
+		width -= 1; height -= 1;
+
+		crp = cairo_pattern_create_linear (x, y, x, y + height);
+		cairo_pattern_add_color_stop_rgb (crp, 0.0, grad_top.r, grad_top.g, grad_top.b);
+		cairo_pattern_add_color_stop_rgb (crp, 0.14, grad_top.r, grad_top.g, grad_top.b);
+		cairo_pattern_add_color_stop_rgb (crp, 0.86, grad_bot.r, grad_bot.g, grad_bot.b);
+		cairo_pattern_add_color_stop_rgb (crp, 1.0, grad_bot.r, grad_bot.g, grad_bot.b);
+		cairo_set_source (cr, crp);
+
+		cairo_rectangle (cr, x, y, width, height);
+		cairo_fill (cr);
+		cairo_pattern_destroy (crp);
 	}
 }
 
@@ -351,74 +392,38 @@ paint_shadow (cairo_t *cr, GtkStyle *style,
 	ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 	ge_shade_color (&bg, OUTLINE_SHADE, &border); /* border */
 
-	x += 0.5; y += 0.5;
-	width -= 1.0; height -= 1.0;
+	x += 0.5; y += 0.5; width--; height--;
 
-	/* outer rectangle */
-	if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
+	CairoColor tl, bl;
+	tl.r = 1; tl.g = 1; tl.b = 1; tl.a = 0.7;
+	bl.r = 0; bl.g = 0; bl.b = 0; bl.a = 0.2;
+
+	if (shadow_type == GTK_SHADOW_OUT)
 	{
 		ge_cairo_set_color (cr, &border);
 		ge_cairo_stroke_rectangle (cr, x, y, width, height);
+		x++; y++; width -= 2.0; height -= 2.0;
+		ge_cairo_simple_border (cr, &tl, &bl, x, y, width +1, height +1, TRUE);
 	}
-	else if (shadow_type == GTK_SHADOW_IN || shadow_type == GTK_SHADOW_ETCHED_IN)
+	else if (shadow_type == GTK_SHADOW_IN)
 	{
-		/*
-		cairo_pattern_add_color_stop_rgb (crp, 0.0, 186/255.0, 189/255.0, 182/255.0);
-		cairo_pattern_add_color_stop_rgb (crp, 1.0, 238/255.0, 238/255.0, 236/255.0);
-		*/
-		crp = cairo_pattern_create_linear (x, y, x, y + height);
-		cairo_pattern_add_color_stop_rgba (crp, 1.0, 1.0, 1.0, 1.0, 0.5);
-		cairo_pattern_add_color_stop_rgba (crp, 0.0, .0, .0, .0, 0.2);
-		cairo_set_source (cr, crp);
-		ge_cairo_stroke_rectangle (cr, x, y, width, height);
-		cairo_pattern_destroy (crp);
-	}
-
-	/* inner rectangle */
-	x += 1.0; y += 1.0;
-	width -= 2.0; height -= 2.0;
-
-	/* stroke */
-	if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
-	{
-		crp = cairo_pattern_create_linear (x, y, x, y + height);
-		/*
-		cairo_pattern_add_color_stop_rgb (crp, 0.0, 1.0, 1.0, 1.0);
-		cairo_pattern_add_color_stop_rgb (crp, 1.0, 136/255.0, 138/255.0, 133/255.0);
-		*/
-		cairo_pattern_add_color_stop_rgba (crp, 0.0, 1.0, 1.0, 1.0, 0.5);
-		cairo_pattern_add_color_stop_rgba (crp, 1.0, .0, .0, .0, 0.2);
-		cairo_set_source (cr, crp);
-		ge_cairo_stroke_rectangle (cr, x, y, width, height);
-		cairo_pattern_destroy (crp);
-	}
-	else if (shadow_type == GTK_SHADOW_IN || shadow_type == GTK_SHADOW_ETCHED_IN)
-	{
-		/*TODO: Find a way to calculate this value (same as shadow out outer line)*/
+		ge_cairo_simple_border (cr, &bl, &tl, x, y, width +1, height +1, TRUE);
+		x++; y++; width -= 2.0; height -= 2.0;
 		ge_cairo_set_color (cr, &border);
 		ge_cairo_stroke_rectangle (cr, x, y, width, height);
-
-		/*
-		// three lines of alpha: 0.22, 0.12, 0.03
-		cairo_move_to (cr, x + 1.0, y + height);
-		cairo_line_to (cr, x + 1.0, y + 1.0);
-		cairo_line_to (cr, x + width - 1.0, y + 1.0);
-		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.22);
-		cairo_stroke (cr);
-
-		cairo_move_to (cr, x + 2.0, y + height);
-		cairo_line_to (cr, x + 2.0, y + 2.0);
-		cairo_line_to (cr, x + width - 1.0, y + 2.0);
-		cairo_line_to (cr, x + width - 1.0, y + height - 1.0);
-		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.12);
-		cairo_stroke (cr);
-
-		cairo_rectangle (cr, x + 3.0, y + 3.0, width - 5.0, height - 4.0);
-		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.03);
-		cairo_stroke (cr);
-		*/
 	}
-
+	else if (shadow_type == GTK_SHADOW_ETCHED_IN)
+	{
+		ge_cairo_simple_border (cr, &bl, &tl, x, y, width +1, height +1, TRUE);
+		x++; y++; width -= 2.0; height -= 2.0;
+		ge_cairo_simple_border (cr, &tl, &bl, x, y, width +1, height +1, TRUE);
+	}
+	else if (shadow_type == GTK_SHADOW_ETCHED_OUT)
+	{
+		ge_cairo_simple_border (cr, &tl, &bl, x, y, width +1, height +1, TRUE);
+		x++; y++; width -= 2.0; height -= 2.0;
+		ge_cairo_simple_border (cr, &bl, &tl, x, y, width +1, height +1, TRUE);
+	}
 
 }
 
@@ -737,44 +742,28 @@ draw_hline (GtkStyle *style,
 	    const gchar *detail,
 	    gint x1, gint x2, gint y)
 {
-    gint thickness_light;
-    gint thickness_dark;
-    gint i;
+	CairoColor base, light, dark;
+	cairo_t *cr;
 
-    CHECK_ARGS
+	CHECK_ARGS
 
-    debug ("draw_hline: detail=%s state=%d x1=%d x2=%d y=%d\n",
-	    detail, state_type, x1, x2, y);
+	debug ("draw_hline: detail=%s state=%d x1=%d x2=%d y=%d\n",
+		detail, state_type, x1, x2, y);
 
-    thickness_light = style->ythickness / 2;
-    thickness_dark = style->ythickness - thickness_light;
-
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (style->light_gc[state_type], area);
-	gdk_gc_set_clip_rectangle (style->dark_gc[state_type], area);
-    }
-    for (i = 0; i < thickness_dark; i++)
-    {
-	gdk_draw_line (window, style->light_gc[state_type], x2 - i - 1,
-		       y + i, x2, y + i);
-	gdk_draw_line (window, style->dark_gc[state_type], x1, y + i,
-		       x2 - i - 1, y + i);
-    }
-
-    y += thickness_dark;
-    for (i = 0; i < thickness_light; i++)
-    {
-	gdk_draw_line (window, style->dark_gc[state_type], x1, y + i,
-		       x1 + thickness_light - i - 1, y + i);
-	gdk_draw_line (window, style->light_gc[state_type],
-		       x1 + thickness_light - i - 1, y + i, x2, y + i);
-    }
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (style->light_gc[state_type], NULL);
-	gdk_gc_set_clip_rectangle (style->dark_gc[state_type], NULL);
-    }
+	cr = ge_gdk_drawable_to_cairo (window, area);
+	ge_gdk_color_to_cairo (&style->bg[state_type], &base);
+	ge_shade_color (&base, 0.88, &dark);
+	ge_shade_color (&base, 1.12, &light);
+	ge_cairo_set_color (cr, &dark);
+	cairo_move_to (cr, x1 + 0.5, y + 0.5);
+	cairo_line_to (cr, x2 + 0.5, y + 0.5);
+	cairo_stroke (cr);
+	y++;
+	ge_cairo_set_color (cr, &light);
+	cairo_move_to (cr, x1 + 0.5, y + 0.5);
+	cairo_line_to (cr, x2 + 0.5, y + 0.5);
+	cairo_stroke (cr);
+	cairo_destroy (cr);
 }
 
 static void
@@ -784,44 +773,28 @@ draw_vline (GtkStyle *style,
 	    GdkRectangle *area,
 	    GtkWidget *widget, const gchar *detail, gint y1, gint y2, gint x)
 {
-    gint thickness_light;
-    gint thickness_dark;
-    gint i;
+	CairoColor base, light, dark;
+	cairo_t *cr;
 
-    CHECK_ARGS
+	CHECK_ARGS
 
-    debug ("draw_vline: detail=%s state=%d x=%d y1=%d y2=%d\n",
-	    detail, state_type, x, y1, y2);
+	debug ("draw_vline: detail=%s state=%d x=%d y1=%d y2=%d\n",
+		detail, state_type, x, y1, y2);
 
-    thickness_light = style->xthickness / 2;
-    thickness_dark = style->xthickness - thickness_light;
-
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (style->light_gc[state_type], area);
-	gdk_gc_set_clip_rectangle (style->dark_gc[state_type], area);
-    }
-    for (i = 0; i < thickness_dark; i++)
-    {
-	gdk_draw_line (window, style->light_gc[state_type], x + i,
-		       y2 - i - 1, x + i, y2);
-	gdk_draw_line (window, style->dark_gc[state_type], x + i, y1, x + i,
-		       y2 - i - 1);
-    }
-
-    x += thickness_dark;
-    for (i = 0; i < thickness_light; i++)
-    {
-	gdk_draw_line (window, style->dark_gc[state_type], x + i, y1, x + i,
-		       y1 + thickness_light - i);
-	gdk_draw_line (window, style->light_gc[state_type], x + i,
-		       y1 + thickness_light - i, x + i, y2);
-    }
-    if (area)
-    {
-	gdk_gc_set_clip_rectangle (style->light_gc[state_type], NULL);
-	gdk_gc_set_clip_rectangle (style->dark_gc[state_type], NULL);
-    }
+	cr = ge_gdk_drawable_to_cairo (window, area);
+	ge_gdk_color_to_cairo (&style->bg[state_type], &base);
+	ge_shade_color (&base, 0.88, &dark);
+	ge_shade_color (&base, 1.12, &light);
+	ge_cairo_set_color (cr, &dark);
+	cairo_move_to (cr, x + 0.5, y1 + 0.5);
+	cairo_line_to (cr, x + 0.5, y2 + 0.5);
+	cairo_stroke (cr);
+	x++;
+	ge_cairo_set_color (cr, &light);
+	cairo_move_to (cr, x + 0.5, y1 + 0.5);
+	cairo_line_to (cr, x + 0.5, y2 + 0.5);
+	cairo_stroke (cr);
+	cairo_destroy (cr);
 }
 
 static void
@@ -872,6 +845,22 @@ draw_shadow (GtkStyle *style,
 		if (GE_IS_WIDGET (button))
 			gtk_widget_queue_draw_area (button,  button->allocation.x, button->allocation.y, button->allocation.width,button->allocation.height);
 	}
+
+	if (widget && GTK_IS_SPIN_BUTTON (widget))
+	{
+		GtkWidget *button;
+		if (ge_widget_is_ltr (widget))
+			width += 2;
+		else
+		{
+			x -= 3;
+			width += 3;
+		}
+
+		if (area == NULL)
+			area = &area2;
+	}
+	
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
 	if (DETAIL ("entry"))
@@ -900,48 +889,69 @@ draw_box (GtkStyle *style,
     debug ("draw_box: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
+	cr = ge_gdk_drawable_to_cairo (window, area);
+
 	if (DETAIL ("spinbutton"))
 	{
-		/* this is a border round spin buttons - we don't want anything here */
-		gtk_style_apply_default_background (style, window, TRUE, state_type, area, x, y, width, height);
+		gboolean focused = FALSE;
+		GtkWidget *entry;
+		if (widget) {
+			state_type = GTK_WIDGET_STATE (widget);
+			focused = GTK_WIDGET_HAS_FOCUS (widget);
+		}
+		width += 3;
+		if (ge_widget_is_ltr (widget))
+			x -= 3;
+
+		paint_entry_shadow (cr, style, state_type, focused, x, y, width, height);
+
+		cairo_destroy (cr);
 		return;
 	}
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	/* no pressed state for scollbar buttons yet... */
 	if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar"))
 		shadow_type = GTK_SHADOW_OUT;
 
 	if (DETAIL ("button") || DETAIL ("optionmenu") || DETAIL ("spinbutton_down") || DETAIL ("spinbutton_up"))
 	{
+		gboolean extra_shadow = TRUE;
+		gboolean focused = FALSE;
+
+		if (widget)
+			focused = GTK_WIDGET_HAS_FOCUS (widget);
+
 		if (DETAIL ("spinbutton_down") || DETAIL ("spinbutton_up"))
 		{
 			/* add some padding */
-			x++; y++; width -= 2;
-			if (DETAIL ("spinbutton_up")) 
-				height--;
+			width -= 3;
+			if (DETAIL ("spinbutton_up"))
+			{
+				y += 3;
+				height -= 3;
+			}
 			else
-				height -= 2;
-
-			if (state_type == GTK_STATE_INSENSITIVE)
-				height++;
-
+			{
+				y += 1;
+				height -= 4;
+			}
+			if (!ge_widget_is_ltr (widget))
+				x += 3;
+			extra_shadow = FALSE;
 		}
 
 		if (widget && (GE_IS_COMBO (widget->parent) || GE_IS_COMBO_BOX_ENTRY (widget->parent)))
 		{
 			/* Combobox buttons */
-			gboolean focused = FALSE;
 			GtkWidget *entry;
-			/* TODO: need RTL support */
+			gboolean entry_focused = FALSE;
+
 			if ((entry = g_object_get_data ((GObject*) widget->parent, "entry")))
 			{
-				focused = (GTK_WIDGET_HAS_FOCUS (entry));
+				entry_focused = (GTK_WIDGET_HAS_FOCUS (entry));
 				state_type = GTK_WIDGET_STATE (entry);
 			}
 
-
 			if (state_type == GTK_STATE_INSENSITIVE)
 				gdk_cairo_set_source_color (cr, &style->base[GTK_STATE_INSENSITIVE]);
 			else
@@ -952,15 +962,13 @@ draw_box (GtkStyle *style,
 			g_object_set_data ((GObject*) widget->parent, "button", widget);
 
 			if (ge_widget_is_ltr (widget))
-				paint_entry_shadow (cr, style, state_type, focused, x - 4, y, width + 4, height);
+				paint_entry_shadow (cr, style, state_type, entry_focused, x - 4, y, width + 4, height);
 			else
-				paint_entry_shadow (cr, style, state_type, focused, x, y, width + 4, height);
-
-			if (state_type == GTK_STATE_INSENSITIVE)
-				height++;
+				paint_entry_shadow (cr, style, state_type, entry_focused, x, y, width + 4, height);
 
 			x += 3; y += 3;
 			width -= 6; height -= 6;
+			extra_shadow = FALSE;
 		}
 		if (widget && (GE_IS_TREE_VIEW (widget->parent)))
 		{
@@ -974,18 +982,21 @@ draw_box (GtkStyle *style,
 					
 			x += 1; y += 1;
 			width -= 2; height -= 2;
+			extra_shadow = FALSE;
 		}
-		paint_button (cr, style, state_type, shadow_type, x, y, width, height);
+		paint_button (cr, style, state_type, shadow_type, focused, extra_shadow, x, y, width, height);
 	}
 	else if (DETAIL ("buttondefault"))
 	{
 		/* draw Default Button marking, but only when sensitive */
-		if (widget && state_type != GTK_STATE_INSENSITIVE)
+		if (state_type != GTK_STATE_INSENSITIVE)
 		{
-			gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_SELECTED]);
-			cx = x + 0.5; cy = y + 0.5; cw = width - 1.0; ch = height - 1.0;
-			ge_cairo_rounded_rectangle (cr, cx, cy, cw, ch, 3.0, CR_CORNER_ALL);
-			cairo_stroke (cr);
+			CairoColor c;
+			ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &c);
+			cairo_set_source_rgba (cr, c.r, c.g, c.b, 0.6);
+
+			ge_cairo_rounded_rectangle (cr, x, y, width, height, 3.0, CR_CORNER_ALL);
+			cairo_fill (cr);
 		}
 	}
 	else if (DETAIL ("menuitem"))
@@ -1052,9 +1063,40 @@ draw_box (GtkStyle *style,
 		/* fill  */
 		cairo_rectangle (cr, x, y, width, height);
 
-		if (DETAIL ("toolbar") || DETAIL ("menubar") || DETAIL ("vscrollbar")
-				|| DETAIL ("hscrollbar") || DETAIL ("handlebox") || DETAIL ("dockitem"))
+		if (DETAIL ("toolbar") || DETAIL ("menubar")
+			|| DETAIL ("vscrollbar") || DETAIL ("hscrollbar")
+			|| DETAIL ("handlebox_bin") || DETAIL ("dockitem"))
 		{
+			/* make the handle and child widget appear as one */
+			if (widget && widget->parent && GTK_IS_HANDLE_BOX (widget->parent))
+			{
+				GtkPositionType position;
+				position = gtk_handle_box_get_handle_position (GTK_HANDLE_BOX (widget->parent));
+				if (!ge_widget_is_ltr (widget->parent))
+				{
+					if (position == GTK_POS_TOP) position = GTK_POS_BOTTOM;
+					else if (position == GTK_POS_BOTTOM) position = GTK_POS_TOP;
+					else if (position == GTK_POS_LEFT) position = GTK_POS_RIGHT;
+					else if (position == GTK_POS_RIGHT) position = GTK_POS_LEFT;
+				}
+				switch (position)
+				{
+					case GTK_POS_TOP:
+						y -= 3;
+						height += 3;
+						break;
+					case GTK_POS_BOTTOM:
+						height += 3;
+						break;
+					case GTK_POS_LEFT:
+						x -= 3;
+						width += 3;
+						break;
+					case GTK_POS_RIGHT:
+						width += 3;
+				}
+			}
+
 			if (shadow_type == GTK_SHADOW_OUT || shadow_type == GTK_SHADOW_ETCHED_OUT)
 			{
 				cairo_pattern_t *crp;
@@ -1162,8 +1204,12 @@ draw_arrow (GtkStyle *style,
 
 	if (DETAIL ("spinbutton"))
 	{
-		if (arrow_type == GTK_ARROW_DOWN)
-			y+=2;
+		if (ge_widget_is_ltr (widget))
+			x--;
+		else
+			x += 1;
+		if (arrow_type == GTK_ARROW_UP)
+			y++;
 	}
 	if (arrow_type == GTK_ARROW_UP || arrow_type == GTK_ARROW_DOWN)
 	{
@@ -1532,7 +1578,7 @@ draw_shadow_gap (GtkStyle *style,
 	break;
     }
 
-    gtk_style_apply_default_pixmap (style, window, state_type, area,
+    gtk_style_apply_default_background (style, window, TRUE, state_type, area,
 				    rect.x, rect.y, rect.width, rect.height);
 }
 
@@ -1555,62 +1601,43 @@ draw_box_gap (GtkStyle *style,
     CHECK_ARGS
     SANITIZE_SIZE
 
-    debug ("draw_box_gap: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
-	    detail, state_type, shadow_type, x, y, width, height);
+	debug ("draw_box_gap: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
+		detail, state_type, shadow_type, x, y, width, height);
 
-    gtk_paint_box (style, window, state_type, shadow_type, area, widget,
-		   detail, x, y, width, height);
-
-    /* XXX Eavel hack to prevent a hole being draw when the
-       XXX active tab is on the far left */
-    if (gap_x < 1)
-    {
-	gap_width -= (1 - gap_x);
-	gap_x = 1;
-    }
-    switch (gap_side)
-    {
-    case GTK_POS_TOP:
-    case GTK_POS_BOTTOM:
-      gap_width = MIN (gap_width, width - gap_x - 1);
-      break;
-    case GTK_POS_LEFT:
-    case GTK_POS_RIGHT:
-      gap_width = MIN (gap_width, height - gap_x - 1);
-      break;
-    }
+	gtk_paint_box (style, window, state_type, shadow_type, area, widget,
+			detail, x, y, width, height);
 
 
-    switch (gap_side)
-    {
-    case GTK_POS_TOP:
-	rect.x = x + gap_x;
-	rect.y = y;
-	rect.width = gap_width - 1;
-	rect.height = 2;
-	break;
-    default:
-    case GTK_POS_BOTTOM:
-	rect.x = x + gap_x;
-	rect.y = y + height - 2;
-	rect.width = gap_width - 1;
-	rect.height = 2;
-	break;
-    case GTK_POS_LEFT:
-	rect.x = x;
-	rect.y = y + gap_x;
-	rect.width = 2;
-	rect.height = gap_width - 1;
-	break;
-    case GTK_POS_RIGHT:
-	rect.x = x + width - 2;
-	rect.y = y + gap_x;
-	rect.width = 2;
-	rect.height = gap_width - 1;
-	break;
-    }
+	switch (gap_side)
+	{
+		case GTK_POS_TOP:
+			rect.x = x + gap_x + 1;
+			rect.y = y;
+			rect.width = gap_width - 2;
+			rect.height = 2;
+			break;
+		default:
+		case GTK_POS_BOTTOM:
+			rect.x = x + gap_x + 1;
+			rect.y = y + height - 2;
+			rect.width = gap_width - 2;
+			rect.height = 2;
+			break;
+		case GTK_POS_LEFT:
+			rect.x = x;
+			rect.y = y + gap_x + 1;
+			rect.width = 2;
+			rect.height = gap_width - 2;
+			break;
+		case GTK_POS_RIGHT:
+			rect.x = x + width - 2;
+			rect.y = y + gap_x + 1;
+			rect.width = 2;
+			rect.height = gap_width - 2;
+			break;
+	}
 
-    gtk_style_apply_default_pixmap (style, window, state_type, area,
+	gtk_style_apply_default_background (style, window, TRUE, state_type, area,
 				    rect.x, rect.y, rect.width, rect.height);
 }
 
@@ -1629,6 +1656,7 @@ draw_extension (GtkStyle *style,
 	cairo_pattern_t *crp;
 	CairoColor c1, c2, bg, outline;
 	CairoCorners corners;
+	gdouble radius = 2;
 
 	debug ("draw_extension: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 		detail, state_type, shadow_type, x, y, width, height);
@@ -1636,18 +1664,17 @@ draw_extension (GtkStyle *style,
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
 	/* set up colours */
-	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &bg);
+	ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 	ge_shade_color (&bg, 1.1, &c1);
 	c2.r = bg.r; c2.g = bg.g; c2.b = bg.b;
+
+	/* make sure the correct area is clipped */
+	cairo_rectangle (cr, x, y, width, height);
+	cairo_clip (cr);
+
 	/* outline colour */
 	ge_shade_color (&bg, OUTLINE_SHADE, &outline);
 
-	cairo_save (cr); /* save initial clip for later */
-	cairo_rectangle (cr, x, y, width, height);
-	cairo_clip_preserve (cr);
-	cairo_new_path (cr);
-
-
 	switch (gap_side)
 	{
 		case GTK_POS_TOP:
@@ -1680,7 +1707,7 @@ draw_extension (GtkStyle *style,
 			break;
 	}
 
-	ge_cairo_rounded_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0, 2.0, corners);
+	ge_cairo_rounded_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0, radius, corners);
 
 	cairo_pattern_add_color_stop_rgb (crp, 0.0, c1.r, c1.g, c1.b);
 	cairo_pattern_add_color_stop_rgb (crp, 0.3, c2.r, c2.g, c2.b);
@@ -1690,30 +1717,90 @@ draw_extension (GtkStyle *style,
 
 	ge_cairo_set_color (cr, &outline);
 	cairo_stroke (cr);
-	
 
 	x++; y++; width -= 2.0; height -= 2.0;
+	radius--;
+
+	/* setup for stroke */
+	cairo_translate (cr, 0.5, 0.5);
+	width--; height--;
+
 	if (state_type == GTK_STATE_NORMAL)
 	{
-		cairo_restore (cr); /* restore the original clip */
+		/* we want to draw outside the width/height now */
+		cairo_reset_clip (cr);
+
 		switch (gap_side)
 		{
-			case GTK_POS_BOTTOM: height += 2; cairo_rectangle (cr, x, y, width, height -1); break; /* top */
-			case GTK_POS_TOP: y -= 2; height += 2; cairo_rectangle (cr, x, y + 1, width, height - 1); break; /* bottom */
-			case GTK_POS_LEFT: x -= 2; width += 2; cairo_rectangle (cr, x + 1, y, width - 1, height); break; /* right */
-			case GTK_POS_RIGHT: width += 2; cairo_rectangle (cr, x, y, width - 1, height); break; /* left */
+			/* top */
+			case GTK_POS_BOTTOM:
+				height++;
+				cairo_move_to (cr, x, y + height);
+				cairo_arc (cr, x + radius, y + radius, radius, M_PI, M_PI * 1.5);
+				cairo_line_to (cr, x + width - radius, y);
+				cairo_set_source_rgba (cr, 1, 1, 1, 0.7);
+				cairo_stroke (cr);
+
+				cairo_arc (cr, x + width - radius, y + radius, radius, M_PI * 1.5, 0);
+				cairo_line_to (cr, x + width, y + height - 1);
+				cairo_set_source_rgba (cr, 0, 0, 0, 0.2);
+				cairo_stroke (cr);
+				break;
+			/* bottom */
+			case GTK_POS_TOP:
+				y -= 1; height += 1;
+				cairo_move_to (cr, x, y);
+				cairo_arc_negative (cr, x + radius, y + height - radius, radius, M_PI, M_PI * 0.5);
+				cairo_set_source_rgba (cr, 1, 1, 1, 0.7);
+				cairo_stroke (cr);
+
+				cairo_move_to (cr, x + 1, y + height);
+				cairo_line_to (cr, x + width - radius, y + height);
+				cairo_arc_negative (cr, x + width - radius, y + height - radius, radius, M_PI * 0.5, 0);
+				cairo_line_to (cr, x + width, y);
+				cairo_set_source_rgba (cr, 0, 0, 0, 0.2);
+				cairo_stroke (cr);
+				break;
+			/* right */
+			case GTK_POS_LEFT:
+				x -= 1; width += 1;
+				cairo_move_to (cr, x, y);
+				cairo_arc (cr, x + width - radius, y + radius, radius, M_PI * 1.5, 0);
+				cairo_arc (cr, x + width - radius, y + height - radius, radius, 0, M_PI * 0.5);
+				cairo_set_source_rgba (cr, 1, 1, 1, 0.7);
+				cairo_stroke (cr);
+				
+				cairo_move_to (cr, x + width - 1, y + height);
+				cairo_line_to (cr, x, y + height);
+				cairo_set_source_rgba (cr, 0, 0, 0, 0.2);
+				cairo_stroke (cr);
+				break;
+			/* left */
+			case GTK_POS_RIGHT:
+				width += 1;
+				cairo_move_to (cr, x + width, y);
+				cairo_arc_negative (cr, x + radius, y + radius, radius, M_PI * 1.5, M_PI);
+				cairo_arc_negative (cr, x + radius, y + height - radius, radius, M_PI, M_PI * 0.5);
+				cairo_set_source_rgba (cr, 1, 1, 1, 0.7);
+				cairo_stroke (cr);
+
+				cairo_move_to (cr, x + 1, y + height);
+				cairo_line_to (cr, x + width, y + height);
+				cairo_set_source_rgba (cr, 0, 0, 0, 0.2);
+				cairo_stroke (cr);
+				break;
 		}
 		
-		cairo_clip_preserve (cr);
-		cairo_new_path (cr);
+	}
+	else
+	{
+		ge_cairo_rounded_rectangle (cr, x, y, width, height, 1.0, corners);
+		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5); 
+		cairo_stroke (cr);
 	}
 
-	ge_cairo_rounded_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0, 1.0, corners);
-	if (gap_side == GTK_POS_TOP && state_type == GTK_STATE_NORMAL)
-		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2);
-	else
-		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5); /* must be same colour as corresponding draw_box highlight */
-	cairo_stroke (cr);
+
+
 	cairo_destroy (cr);
 }
 
@@ -1738,6 +1825,11 @@ draw_focus (GtkStyle *style,
 		cairo_destroy (cr);
 		return;
 	}
+
+	/* buttons draw their own focus indicator */
+	if (DETAIL ("button"))
+		return;
+
 	/* Make sure no widget is without a focus indicator! */
 	parent_style_class->draw_focus(style, window, state_type, area, widget,
 	                               detail, x, y, width, height);
@@ -1852,14 +1944,11 @@ draw_handle (GtkStyle *style,
 	state_type = GTK_STATE_ACTIVE;
 
 
-	if (!ge_is_panel_widget_item (widget) && !ge_object_is_a ((GObject*)widget, "PanelToplevel"))
-		draw_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
-
     light_gc = style->light_gc[state_type];
     dark_gc = style->dark_gc[state_type];
 
-    xthick = style->xthickness;
-    ythick = style->ythickness;
+    xthick = style->xthickness + 1;
+    ythick = style->ythickness + 1;
 
     dest.x = x + xthick;
     dest.y = y + ythick;
