@@ -1,4 +1,7 @@
+#ifndef __GENERAL_SUPPORT_H
+#define __GENERAL_SUPPORT_H
 
+#include <gmodule.h>
 #include <glib.h>
 
 /* macros to make sure that things are sane ... */
@@ -20,16 +23,21 @@
   else if (height == -1)				\
     gdk_drawable_get_size (window, NULL, &height);
 
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 3)) && defined(__ELF__)
-# define GE_EXPORT      __attribute__((__visibility__("default")))
-# define GE_HIDDEN      __attribute__((__visibility__("hidden")))
-# define GE_INTERNAL    __attribute__((__visibility__("internal")))
-#elif defined(__SUNPRO_C) && (__SUNPRO_C >= 0x550)
+#define GE_EXPORT	G_MODULE_EXPORT
+#define GE_INTERNAL	G_GNUC_INTERNAL
+
+/* explicitly export with ggc, G_MODULE_EXPORT does not do this, this should
+ * make it possible to compile with -fvisibility=hidden */
+#ifdef G_HAVE_GNUC_VISIBILITY
+# undef GE_EXPORT
+# define GE_EXPORT	__attribute__((__visibility__("default")))
+#endif
+
+#if defined(__SUNPRO_C) && (__SUNPRO_C >= 0x550)
+# undef GE_EXPORT
+# undef GE_INTERNAL
 # define GE_EXPORT      __global
-# define GE_HIDDEN      __hidden
 # define GE_INTERNAL    __hidden
-#else /* not gcc >= 3.3 and not Sun Studio >= 8 */
-# define GE_EXPORT
-# define GE_HIDDEN
-# define GE_INTERNAL
 #endif 
+
+#endif /* __GENERAL_SUPPORT_H */
